/**
 * @module dixim
 * @submodule common
 */
dixim.namespace('dixim.common');

/**
 * @module common
 * @submodule api-client-transfer
 */
(function() {
var ApiClientTransfer;
var prototype;
var suprtConstructor;
var ApiClient = dixim.common.ApiClient;
var Configs = dixim.ctt.Configs;

/**
 * Provides interfaces to transfer API.
 *
 * This API client supports two type of event handling method,
 * callback and event-subscription.
 *
 * The callback method can be specified as the parameter of each method.
 * (Such as request.success, request.error or request.complete.)
 * When the callback is called, an Event instance which is related to
 * the triggered method is passed to the callback.
 *
 * This API client is also an EventTarget, you can subscribe an event
 * by addEventListener().
 *
 * @class ApiClientTransfer
 * @constructor
 * @extends EventTarget
 * @param {Object} [configs]
 */
ApiClientTransfer = function(configs) {
    ApiClient.call(this, configs);
};

dixim.inherit(ApiClientTransfer, ApiClient);
dixim.common.ApiClientTransfer = ApiClientTransfer;
prototype = ApiClientTransfer.prototype;
superConstructor = ApiClientTransfer.suprtConstructor;

/**
 * Fetch storage list.
 *
 * @method fetchStorageList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchStorageList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchStorageList', [Configs.get('api'), '/storage_list'].join(''), request, 'POST');
};

/**
 * Browse contents.
 *
 * @method browse
 * @param {Object} request
 * @param {String} [request.id]
 * @param {String} [request.browse_flag]
 * @param {Integer} [request.starting_index]
 * @param {Integer} [request.requested_count]
 * @param {String} [request.filter]
 * @param {String} [request.sort_criteria]
 * @param {String} [request.pin_code]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.browse = function(request) {
    var query = {};

    query.id = request.id;
    if (typeof request.browse_flag !== 'undefined') query.browse_flag = request.browse_flag;
    if (typeof request.starting_index !== 'undefined') query.starting_index = request.starting_index;
    if (typeof request.requested_count !== 'undefined') query.requested_count = request.requested_count;
    if (typeof request.filter !== 'undefined') query.filter = request.filter;
    if (typeof request.sort_criteria !== 'undefined') query.sort_criteria = request.sort_criteria;
    if (typeof request.pin_code !== 'undefined') query.pin_code = request.pin_code;

    return this.doSimpleMod.call(this, 'browse', [Configs.get('api'), '/browse'].join(''), query, request);
};

/**
 * Search contents.
 *
 * @method search
 * @param {Object} request
 * @param {String} [request.id]
 * @param {String} [request.key]
 * @param {String} [request.value]
 * @param {Integer} [request.starting_index]
 * @param {Integer} [request.requested_count]
 * @param {String} [request.filter]
 * @param {String} [request.sort_criteria]
 * @param {String} [request.pin_code]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.search = function(request) {
    var query = {};

    query.id = request.id;
    query.key = request.key;
    query.value = request.value.join(',');
    if (typeof request.starting_index !== 'undefined') query.starting_index = request.starting_index;
    if (typeof request.requested_count !== 'undefined') query.requested_count = request.requested_count;
    if (typeof request.filter !== 'undefined') query.filter = request.filter;
    if (typeof request.sort_criteria !== 'undefined') query.sort_criteria = request.sort_criteria;
    if (typeof request.pin_code !== 'undefined') query.pin_code = request.pin_code;

    return this.doSimpleMod.call(this, 'search', [Configs.get('api'), '/search'].join(''), query, request);
};
/**
 * Get token.
 *
 * @method getToken
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.getToken = function(request) {
    return this.doSimpleFetch.call(this, 'getToken', [Configs.get('api'), '/get_token'].join(''), request, 'POST');
};


/**
 * Remove contents.
 *
 * @method remove
 * @param {Object} request
 * @param {String} [request.id]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.remove = function(request) {
    var query = {};

    query.id = request.id;
    query.token = request.token;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'remove', [Configs.get('api'), '/remove'].join(''), query, request);
};

/**
 * Rename contents.
 *
 * @method rename
 * @param {Object} request
 * @param {String} [request.id]
 * @param {String} [request.new_name]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.rename = function(request) {
    var query = {};

    query.id = request.id;
    query.new_name = request.new_name;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'rename', [Configs.get('api'), '/rename'].join(''), query, request);
};

/**
 * Fetch dms list.
 *
 * @method fetchDmsList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDmsList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDmsList', [Configs.get('api'), '/dms_list'].join(''), request, 'POST');
};

/**
 * Remove dms on dms list.
 *
 * @method removeDms
 * @param {Object} request
 * @param {String} [request.udn]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeDms = function(request) {
    var query = {};

    query.udn = request.udn;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeDms', [Configs.get('api'), '/remove_dms'].join(''), query, request);
};

/**
 * Fetch dest storage list.
 *
 * @method fetchDestStorageList
 * @param {Object} request
 * @param {String} [request.udn]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDestStorageList = function(request) {
    var query = {};

    query.udn = request.udn;

    return this.doSimpleMod.call(this, 'fetchDestStorageList', [Configs.get('api'), '/rec_dest_list'].join(''), query, request);
};

/**
 * Transfer contents.
 *
 * @method transfer
 * @param {Object} request
 * @param {String[]} [request.src_ids]
 * @param {String} [request.dst_urn]
 * @param {String} [request.dst_id]
 * @param {Integer} [request.shutdown]
 * @param {String} [request.start_time]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.transfer = function(request) {
    var query = {};

    query.src_ids = request.src_ids.join(',');
    query.dst_udn = request.dst_udn;
    if (typeof request.dst_id !== 'undefined') query.dst_id = request.dst_id;
    if (typeof request.shutdown !== 'undefined') query.shutdown = request.shutdown;
    if (typeof request.start_time !== 'undefined') query.start_time = request.start_time;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'transfer', [Configs.get('api'), '/transfer'].join(''), query, request);
};

/**
 * Fetch transfer status.
 *
 * @method fetchTransferStatus
 * @param {Object} request
 * @param {String} [request.pin_code]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchTransferStatus = function(request) {
    var query = {};

    request = request || {};

    if (typeof request.pin_code !== 'undefined') query.pin_code = request.pin_code;

    return this.doSimpleMod.call(this, 'fetchTransferStatus', [Configs.get('api'), '/transfer_status'].join(''), query, request, 'POST');
};

/**
 * Cancel transfer.
 *
 * @method cancelTransfer
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.cancelTransfer = function(request) {
    var query = {};

    request = request || {};
    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'cancelTransfer', [Configs.get('api'), '/cancel_transfer'].join(''), query, request, 'POST');
};

/**
 * Fetch reservation list.
 *
 * @method fetchReservationList
 * @param {Object} request
 * @param {String} [request.pin_code]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchReservationList = function(request) {
    var query = {};

    if (typeof request.pin_code !== 'undefined') query.pin_code = request.pin_code;

    return this.doSimpleMod.call(this, 'fetchReservationList', [Configs.get('api'), '/reservation_list'].join(''), query, request, 'POST');
};

/**
 * Remove reservation.
 *
 * @method removeReservation
 * @param {Object} request
 * @param {String} [request.id]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeReservation = function(request) {
    var query = {};

    query.id = request.id;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeReservation', [Configs.get('api'), '/remove_reservation'].join(''), query, request, 'POST');
};

/**
 * Change reservation.
 *
 * @method changeReservation
 * @param {Object} request
 * @param {String} [request.id]
 * @param {String} [request.start_time]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.changeReservation = function(request) {
    var query = {};

    query.id = request.id;
    query.start_time = request.start_time;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'changeReservation', [Configs.get('api'), '/update_reservation'].join(''), query, request, 'POST');
};

/**
 * Fetch transfer log.
 *
 * @method fetchTransferLog
 * @param {Object} request
 * @param {String} [request.pin_code]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchTransferLog = function(request) {
    var query = {};

    if (typeof request.pin_code !== 'undefined') query.pin_code = request.pin_code;

    return this.doSimpleMod.call(this, 'fetchTransferLog', [Configs.get('api'), '/log_list'].join(''), query, request, 'POST');
};

/**
 * Clear transfer log.
 *
 * @method clearTransferLog
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.clearTransferLog = function(request) {
    var query = {};

    request = request || {};
    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'clearTransferLog', [Configs.get('api'), '/clear_log_list'].join(''), query, request, 'POST');
};

/**
 * Matching PIN code.
 *
 * @method matchPin
 * @param {Object} request
 * @param {String} [request.code]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.matchPin = function(request) {
    var query = {};

    query.code = request.code;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'matchPin', [Configs.get('api'), '/match_pin_code'].join(''), query, request);
};

/**
 * Set PIN code.
 *
 * @method setPin
 * @param {Object} request
 * @param {String} [request.code]
 * @param {String} [request.code_new]
 * @param {Integer} [request.age_limit]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setPin = function(request) {
    var query = {};

    query.code = request.code;
    query.code_new = request.code_new;
    query.age_limit = request.age_limit;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setPin', [Configs.get('api'), '/set_pin_code'].join(''), query, request);
};

/**
 * Fetch PIN code.
 *
 * @method fetchPin
 * @param {Object} request
 * @param {String} [request.code]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchPin = function(request) {
    var query = {};

    query.code = request.code;

    return this.doSimpleMod.call(this, 'fetchPin', [Configs.get('api'), '/get_pin_code'].join(''), query, request);
};

/**
 * Fetch host name.
 *
 * @method fetchHostName
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchHostName = function(request) {
    return this.doSimpleFetch.call(this, 'fetchHostName', [Configs.get('api'), '/get_host_name'].join(''), request);
};

/**
 * Fetch dn dms list.
 *
 * @method fetchDnDmsList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDnDmsList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDnDmsList', [Configs.get('api'), '/dn_dms_list'].join(''), request, 'POST');
};

/**
 * Browse contents on downloadable device.
 *
 * @method dnBrowse
 * @param {Object} request
 * @param {String} [request.id]
 * @param {String} [request.udn]
 * @param {String} [request.browse_flag]
 * @param {Integer} [request.starting_index]
 * @param {Integer} [request.requested_count]
 * @param {String} [request.pin_code]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.dnBrowse = function(request) {
    var query = {};

    query.id = request.id;
    query.udn = request.udn;
    if (typeof request.browse_flag !== 'undefined') query.browse_flag = request.browse_flag;
    if (typeof request.starting_index !== 'undefined') query.starting_index = request.starting_index;
    if (typeof request.requested_count !== 'undefined') query.requested_count = request.requested_count;
    if (typeof request.pin_code !== 'undefined') query.pin_code = request.pin_code;

    return this.doSimpleMod.call(this, 'dnBrowse', [Configs.get('api'), '/dn_browse'].join(''), query, request);
};

/**
 * Download contents.
 *
 * @method download
 * @param {Object} request
 * @param {String[]} [request.src_ids]
 * @param {String} [request.src_udn]
 * @param {String} [request.dst_id]
 * @param {String} [request.start_time]
 * @param {String} [request.res_criteria]
 * @param {Boolean} [request.shutdown]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.download = function(request) {
    var query = {};

    query.src_ids = request.src_ids.join(',');
    query.src_udn = request.src_udn;
    if (typeof request.dst_id !== 'undefined') query.dst_id = request.dst_id;
    if (typeof request.start_time !== 'undefined') query.start_time = request.start_time;
    if (typeof request.res_criteria !== 'undefined') query.res_criteria = request.res_criteria;
    if (typeof request.shutdown !== 'undefined') query.shutdown = request.shutdown;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'download', [Configs.get('api'), '/dn_transfer'].join(''), query, request, 'POST');
};

/**
 * Send M-SEARCH.
 *
 * @method sendMSearch
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.sendMSearch = function(request) {
    request = request || {};
    request.dataType = 'text';

    return this.doSimpleFetch.call(this, 'sendMSearch', [Configs.get('api'), '/send_m_search'].join(''), request, 'POST');
};

/**
 * Move contents.
 *
 * @method move
 * @param {Object} request
 * @param {String[]} [request.src_ids]
 * @param {String} [request.dst_id]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.move = function(request) {
    var query = {};

    query.src_ids = request.src_ids.join(',');
    query.dst_id = request.dst_id;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'move', [Configs.get('api'), '/move'].join(''), query, request, 'POST');
};

/**
 * Make directoy.
 *
 * @method makeDirectory
 * @param {Object} request
 * @param {String} [request.parent_id]
 * @param {String} [request.name]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.makeDirectory = function(request) {
    var query = {};

    query.parent_id = request.parent_id;
    query.name = request.name;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'makeDirectory', [Configs.get('api'), '/make_directory'].join(''), query, request, 'POST');
};


}());
