/**
 * @module ctt
 * @submodule transfer-progress-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var TransferProgressView = dixim.ctt.TransferProgressView;
var TransferStatusMonitor = dixim.ctt.TransferStatusMonitor;
var TransferStatusListingController = dixim.ctt.TransferStatusListingController;
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;

var TransferProgressController;
var prototype;

/**
 * @class TransferProgressController
 * @constructor
 * @param {Object} configs
 * @param {ContentsTransferApiClient} [configs.model]
 * @param {Boolean} [configs.manualCancel=false] If true, this controller won't cancel the transferring when an user attempted to cancel it, but fires <em>requestCancel</em> event instead.
 * @param {Boolean} [configs.closable=true] It true, this controller shows close button once the transferring has finished.
 */
TransferProgressController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(TransferProgressController, EventTarget);
dixim.ctt.TransferProgressController = TransferProgressController;
prototype = TransferProgressController.prototype;

/**
 * Triggered when the status dialog has been closed.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 * @param {TransferProgressController} event.args.controller
 * @param {Boolean} [event.args.cancelled] true if the transferring is cancelled.
 */
var CLOSE = 'close';

/**
 * Triggered when the transferring has been done.
 * 
 * @event done
 * @param {TransferProgressController} event.args.controller
 */
var DONE = 'done';

/**
 * Triggered when the transferring has been cancelled.
 *
 * @event cancel
 * @param {TransferProgressController} event.args.controller
 */
var CANCEL = 'cancel';

/**
 * Triggered when the cancellation of the transferring is requested.
 *
 * @event requestCancel
 * @param {TransferProgressController} event.args.controller
 */
var REQUEST_CANCEL = 'requestCancel';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {ContentsTransferApiClient} [configs.model]
 * @param {Boolean} [configs.manualCancel=false] If true, this controller won't cancel the transferring when an user attempted to cancel it, but fires <em>requestCancel</em> event instead.
 * @param {Boolean} [configs.closable=true] It true, this controller shows close button once the transferring has finished.
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._doneState = null;
  this._isDone = false;
  this._closable = typeof configs.closable === 'undefined' ? true : configs.closable;
  this._manualCancel = configs.manualCancel;
  this._model = configs.model || new ContentsTransferApiClient();
  this._monitor = new TransferStatusMonitor({
    model: this._model,
    interval: 1000
  });

  this._statusListingController = null;
  this._view = new TransferProgressView({
    showCloseButton: this._closable
  });
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;
  var monitor = this._monitor;

  if (this._closable) {
    v.addEventListener('close', this._didClickClose, this);
  }
  v.addEventListener('cancel', this._didClickCancel, this);
  v.addEventListener('render', this._didRenderView, this);

  monitor.addEventListener('transferring', this._didEnterInProgressState, this);
  monitor.addEventListener('canceling', this._didEnterCancelingState, this);
  monitor.addEventListener('done', this._didEnterDoneState, this);
  monitor.addEventListener('fetch', this._didFetchStatus, this);

  v.show();
  monitor.start();

  if (!!this._statusListingController) {
    this._statusListingController.start(); 
  }
};

prototype._didRenderView = function(e) {
  var slc;

  slc = new TransferStatusListingController({
    containerNode: this._view.getBodyContent()
  });
  slc.start();

  this._statusListingController = slc;
};

prototype._didClickClose = function(e) {
  var args;

  this._stop();
  args = dixim.extend({controller: this}, this._doneState || {});
  this.dispatchEvent(new Event(CLOSE, args));
};

prototype._didClickCancel = function(e) {
  if (!!this._manualCancel) {
    this.dispatchEvent(new Event(REQUEST_CANCEL, {controller: this}));
  } else {
    this._cancel();
  }
};

prototype._didEnterInProgressState = function(e) {
  if (this.isDone) {
    this._view.restartTransfer();
    this._statusListingController.clearRecord();
    this.isDone = false;
  } else {
  this._view.enterInProgressState();
  }
};

prototype._didEnterCancelingState = function(e) {
  this._view.enterCancelingState();
};

prototype._didEnterDoneState = function(e) {
  var args = e.args;
  var ea;

  if (!!args.cancelled) {
    this._view.enterDoneWithCancellationState();
  } else if (!!args.error || !!args.warning) {
    this._view.enterDoneWithErrorState();
  } else {
    this._view.enterDoneState();
  }
  this._doneState = dixim.extend({}, args);

  ea = dixim.extend({controller: this}, args);
  this.isDone = true;
  this.dispatchEvent(new Event(DONE, ea));
};

prototype._didFetchStatus = function(e) {
  if (!!this._statusListingController) {
    this._statusListingController.updateStatus(e.args.data.results);
  }
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._stop();
};

prototype._stop = function() {
  if (!!this._statusListingController) {
    this._statusListingController.stop();
  }
  this._monitor.stop();
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  if (!!this._statusListingController) {
    this._statusListingController.destroy();
  }
  this._monitor.stop();
  this._view.destroy();
};

/**
 * @method cancel
 */
prototype.cancel = function() {
  this._cancel();
};

prototype._cancel = function() {
  var args;

  this._model.cancelTransfer();

  args = {controller: this};
  this.dispatchEvent(new Event(CANCEL, args));
};

}());
