/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule move-dialog-controller
 */
(function() {
var MoveDialogController;
var prototype;
var superConstructor;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var MoveDialogView = dixim.contents.MoveDialogView;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var _ = dixim.ctt.ResourceBundle.get;
var PinManager = dixim.ctt.util.PinManager;

/**
 * @class MoveDialogController
 * @constructor
 * @param {Object} configs
 * @param {MobileTransferApiClient} [configs.model]
 */
MoveDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

var PREV = 'prev';
var UPDATE = 'update';
var SELECT_DIRECTORY = 'selectDirectory';
var MOVE = 'move';

var BROWSING_STATUS = {
    STOPPED: 0,
    STOPPING: 1,
    BROWSING: 2,
};
var ROOT_OBJECTID = 'FS-Folder';

dixim.inherit(MoveDialogController, EventTarget);
dixim.contents.MoveDialogController = MoveDialogController;
prototype = MoveDialogController.prototype;
superConstructor = MoveDialogController.superConstructor;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.targetItem
 * @param {MobileTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._model = new ApiClientTransfer();
    this._model.addEventListener('browse', this._successBrowse, this);
    this._model.addEventListener('browseError', this._errorBrowse, this);
    this._model.addEventListener('move', this._successMove, this);
    this._model.addEventListener('moveError', this._errorMove, this);

    this._view = new MoveDialogView(configs);
    this._view.addEventListener(PREV, this._clickPrev, this);
    this._view.addEventListener(SELECT_DIRECTORY, this._clickDirectory, this);
    this._view.addEventListener(MOVE, this._clickMove, this);

    this._browseRequestData;
    this._browsingStack = [];
    this._browsingStatus = BROWSING_STATUS.STOPPED;
    this._contents = configs.contents;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._browsingStack.push(ROOT_OBJECTID);
    this._browseAfterStop(ROOT_OBJECTID);
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._browse = function(id) {
    this._browseRequest(id);
};

prototype._browseRequest = function(id, startingIndex) {
    var query = {};
    var pin = PinManager.get();

    query.id = id;
    query.starting_index = startingIndex;
    query.requested_count = 20;
    query.filter = 'container';
    if (typeof pin !== 'undefined') query.pin_code = this._pin;
    this._browseRequestData = query;
    this._model.browse(query);
};

prototype._successBrowse = function(e) {
    var results = e.args.results;
    var resultsLen = results.length;
    var totalReturned = e.args.numberReturned;
    var totalMatches = e.args.totalMatches;
    var startingIndex = this._browseRequestData.starting_index;
    var isStopping = (this._browsingStatus === BROWSING_STATUS.STOPPING);
    var isEndBrowsing = ((totalReturned + startingIndex) === totalMatches);
    var isMatchId = true;

    if (resultsLen !== 0) isMatchId = (this._browseRequestData.id === results[0].parentId);

    if (totalMatches === 0) {
        this._view.noRow();
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        return;
    }
    if (isStopping || (!isMatchId)) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        return;
    }

    for (var i = 0; i < resultsLen; i += 1) {
        this._view.addRow(results[i]);
    }

    if (isEndBrowsing) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        return;
    }

    totalReturned += this._browseRequestData.starting_index;
    this._browseRequest(this._browseRequestData.id, totalReturned);
};

prototype._errorBrowse = function() {
    this._view.error(_('contents_move_fetch_error'));
    this._browsingStatus = BROWSING_STATUS.STOPPED;
};

prototype._viewBrowseContents = function(objectId) {
    this._view.cleanList();
    if (objectId === ROOT_OBJECTID) {
        this._view.setReturnButton();
    } else {
        this._view.setReturnButton(PREV);
    }
    this._view.changeCheckbox();
    this._browsingStatus = BROWSING_STATUS.BROWSING;
    this._browseRequest(objectId, 0);
};

prototype._browseAfterStop = function(id) {
    if (this._browsingStatus === BROWSING_STATUS.BROWSING) {
        this._browsingStatus = BROWSING_STATUS.STOPPING;
    }

    if (this._browsingStatus !== BROWSING_STATUS.STOPPED) {
        setTimeout(dixim.proxy(function() {
            this._browseAfterStop(id)
        }, this), 100);
    } else {
        this._view.cleanList();
        this._viewBrowseContents(id);
    }
};

prototype._move = function(dst) {
    var query = {};
    var contentsLen = this._contents.length;

    this._view.startIndicator();
    query.src_ids = [];
    query.dst_id = dst.id;

    for (var i = 0; i < contentsLen; i += 1) {
        query.src_ids.push(this._contents[i].id);
    }
    this._model.move(query);
};

prototype._successMove = function() {
    this._view.stopIndicator();
    this.stop();
    this.dispatchEvent(new Event(UPDATE));
};

prototype._errorMove = function(e) {
    var errorCode = e.args.errorCode;

    this._view.stopIndicator();
    switch(errorCode) {
        case 485:
        case 486:
            this._view.error(_(['contents_move_error_', errorCode].join('')));
            break;
        default:
            this._view.error(_('contents_move_error'));
            break;
    }
};

prototype._clickDirectory = function(e) {
    this._browsingStack.push(e.args.id);
    this._browseAfterStop(e.args.id);
};

prototype._clickMove = function(e) {
    this._move(e.args.dst);
};

prototype._clickPrev = function() {
    var objectId;

    this._browsingStack.pop();
    objectId = this._browsingStack.pop();

    if (typeof objectId === 'undefined') {
        this._browsingStack.push(ROOT_OBJECTID);
        this._browseAfterStop(ROOT_OBJECTID);
    } else {
        this._browsingStack.push(objectId);
        this._browseAfterStop(objectId);
    }
};

}());
